@@ -44,6 +44,8 @@
 #define RESP_OTHER  "TCP misc-options"
 #define RESP_UNKNOWN "unknown"
 
+extern int _thc_ipv6_rawmode;
+
 struct fingerprint {
   char OS[1024];
   char FP[200];
@@ -60,7 +62,6 @@ struct hostent *he = NULL;
 short int si, sp, sp2;
 
 
-
 // all dict entries must start with a single from/to 0,0,0,0
 // and end with a single from/to ffff,ffff,ffff,ffff
 unsigned short int dict_small[] = { 0, 0, 0, 0, /*to */ 0, 0, 0, 0,
@@ -196,7 +197,7 @@ unsigned char tcp_opt[TCP_OPT_LEN] = {
 // cafe, dead, beef, affe, b00b, babe, f00, fefe, ffff, 1337, 666, 0, 1
 
 void help(char *prg) {
-  printf("%s %s (c) 2015 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2017 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf ("Syntax: %s [-CFHLMPSdlpvV] [-I srcip6] [-i file] [-o file] [-e opt] [-s port,..] [-a port,..] [-u port,..] [-T tag] [-W TIME] interface [unicast-or-multicast-address [remote-router]]\n\n", prg);
   printf("Options:\n");
   if (do_help) printf(" Output Options:\n");
@@ -234,6 +235,7 @@ void help(char *prg) {
       printf("  -x port    TCP/UDP src port for -s, -a and -u\n");
       printf("  -Z mac     use given destination mac address\n");
     }
+    printf("  -m         raw mode (for network adapters where you do not have Ethernet)\n");
   }
   if (do_help == 1) {
     printf(" Help Options:\n");
@@ -363,7 +365,7 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
     len -= do_hdr_size;
     if ((ptr[0] & 240) != 0x60)
       return;
-  } else {
+  } else if (_thc_ipv6_rawmode == 0) {
     ptr += 14;
     len -= 14;
   }
@@ -505,7 +507,7 @@ void check_packets(u_char *foo, const struct pcap_pkthdr *header, const unsigned
         ok = 2;
       }
     } else
-      orig_dst = thc_ipv62notation(ptr + 72 + offset);
+      orig_dst = thc_ipv62notation(ptr + 64 + offset);
   }
 
   i = 0;
@@ -803,6 +805,9 @@ int main(int argc, char *argv[]) {
     case 'l':
       prefer = PREFER_LINK;
       break;
+    case 'm':
+      thc_ipv6_rawmode(1);
+      break;
     case 'M':
       enumerate_mac = 1;
       break;
@@ -916,7 +921,7 @@ int main(int argc, char *argv[]) {
     help(argv[0]);
 
   if (slow && ndp_only) {
-    fprintf(stderr, "Error: you can not use the -S and -L options togther!\n");
+    fprintf(stderr, "Error: you can not use the -S and -L options together!\n");
     exit(-1);
   }
 
@@ -1065,7 +1070,7 @@ int main(int argc, char *argv[]) {
 
   if (verbose) {
     timeval = time(NULL);
-    printf("Starting alive6 %s (c) 2015 by van Hauser / THC at %s\n", VERSION, ctime(&timeval));
+    printf("Starting alive6 %s (c) 2017 by van Hauser / THC at %s\n", VERSION, ctime(&timeval));
   }
   if (renew && src6[0] > 0xfd && (multicast6 != NULL && multicast6[0] > 0xfd))
     prefer = PREFER_LINK;
