@@ -11,7 +11,7 @@
 #include "thc-ipv6.h"
 
 void help(char *prg) {
-  printf("%s %s (c) 2015 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2017 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-k | -m mac] interface [target]\n\n", prg);
   printf("Flood the local network with neighbor advertisements.\n");
   printf("Option -k sends with your real src mac, -m with a specified src mac, random for each packet otherwise.\n");
@@ -62,7 +62,9 @@ int main(int argc, char *argv[]) {
     if ((dst = thc_resolve6(argv[2])) == NULL) {
       fprintf(stderr, "Error: invalid target IPv6 address\n");
       exit(-1);
-    }
+    } else {
+      dstmac = thc_get_mac(interface, NULL, dst);
+  }
     if (dst[0] >= 0x20 && dst[0] <= 0xfd)
       prefer = PREFER_GLOBAL;
   }
