@@ -48,7 +48,7 @@ unsigned int method = 0;
 char *interface = NULL, ether[8];
 
 void help(char *prg) {
-  printf("fragrouter6 %s (c) 2015 by %s %s\n\n", VERSION, AUTHOR, RESOURCE);
+  printf("fragrouter6 %s (c) 2017 by %s %s\n\n", VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-dv -f len -e count -t count] INTERFACE EVASION-METHOD\n\n", prg);
   printf("Options:\n");
   printf("  -f len    size of mini fragments (option 64, default %d)\n", minifrag);
@@ -120,7 +120,7 @@ int netfilter_queue_callback(struct nfq_q_handle *hq, struct nfgenmsg *nfmsg, st
       log_message(LOG_DEBUG, "Packet received: %u", id);
 
     // Get payload and ip header
-    len = nfq_get_payload(nfad, (char **) &packet);
+    len = nfq_get_payload(nfad, (unsigned char **) &packet);
     packet_header = (struct ip6_hdr *) packet;
     proto = packet_header->ip6_ctlun.ip6_un1.ip6_un1_nxt;
     
@@ -761,7 +761,6 @@ void signal_manager(int signal) {
   case SIGTERM:
     // Log the signal
     log_message(LOG_NOTICE, "User interrupt!");
-    // Attiva lo switch per l'uscita dal sotware
     exit_from_loop = 1;
     break;
   }
