@@ -12,6 +12,8 @@
 #include <sys/select.h>
 #include "thc-ipv6.h"
 
+int do_fuzzer();
+
 #define COUNT_FLAG     11
 #define COUNT_BYTE      4
 #define COUNT_WORD     16
@@ -233,7 +235,7 @@ char cmsgbuf[1024];
 int fromlen = 0, len, s, t;
 
 void help(char *prg) {
-  printf("%s %s (c) 2015 by %s %s\n", prg, VERSION, "Darrell Ambro", RESOURCE);
+  printf("%s %s (c) 2017 by %s %s\n", prg, VERSION, "Darrell Ambro", RESOURCE);
   printf("Partially based on code by Brandon Hutcheson and Graeme Neilson\n\n");
   printf("Syntax: %s  [-1|-2|-3|-4|-5|-6|-7|-8|-9|-A|-B|-C|-D|-m] -f mac -l link -v ipv6 -x xid -c client -o options interface\n\n", prg);
   printf("Options:\n");
@@ -487,7 +489,7 @@ int fuzz_loop(char *pkt, int *pkt_len) {
   }
 }
 
-dhcpc_listener() {
+void dhcpc_listener() {
   if ((s = thc_bind_udp_port(547)) < 0) {
     fprintf(stderr, "Error: could not bind to 547/udp\n");
     exit(-1);
@@ -676,7 +678,7 @@ int main(int argc, char *argv[]) {
   }
 
   if (test_end < test_start) {
-    printf("dont fuck up the command line options!\n");
+    printf("don't fuck up the command line options!\n");
     exit(-1);
   }
 
@@ -719,6 +721,7 @@ int main(int argc, char *argv[]) {
   printf("\nFields Fuzzed:\n\n");
   do_fuzzer();
 }
+
 int do_fuzzer() {
   char *ptr;
 
