@@ -21,11 +21,11 @@ extern int do_6in4;
 extern int do_hdr_vlan;
 
 void help(char *prg, int help) {
-  printf("%s %s (c) 2015 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
+  printf("%s %s (c) 2017 by %s %s\n\n", prg, VERSION, AUTHOR, RESOURCE);
   printf("Syntax: %s [-EafqxO] [-e ethertype] [-H t:l:v] [-D t:l:v] [-F dst] [-e ethertype] [-L length] [-N nextheader] [-V version] [-t ttl] [-c class] [-l label] [-d size] [-S port|-U port|-T type -C code] interface src6 dst6 [srcmac [dstmac [data]]]\n\n", prg);
   printf("Options:\n");
   if (help) {
-    printf("  -x              flood mode (doesnt check for replies)\n");
+    printf("  -x              flood mode (doesn't check for replies)\n");
     printf("  -a              add a hop-by-hop header with router alert option.\n");
     printf("  -q              add a hop-by-hop header with quickstart option.\n");
     printf("  -E              send as ethertype IPv4\n");
@@ -145,11 +145,12 @@ void check_packets(u_char *pingdata, const struct pcap_pkthdr *header, const uns
         printf("icmp6 %d:%d", ptr[40 + offset], ptr[41 + offset]);
         resp_type = 0;
       }
+      printf("\n");
     } else
       printf("(ignoring icmp6 packet with different contents (proto %d, type %d, code %d)) ", ptr[nxt], ptr[40 + offset], ptr[41 + offset]);
   } else {
     if (type == NXT_TCP && ptr[nxt] == NXT_TCP) {
-      printf("%04u.%04ld \ttcp-", (int) (ts2.tv_sec - ts.tv_sec - min), usec);
+      printf("%04u.%03ld \ttcp-", (int) (ts2.tv_sec - ts.tv_sec - min), usec);
       switch ((ptr[53 + offset] % 8)) {
       case 2:
         if (ptr[53 + offset] >= TCP_ACK) {
@@ -182,7 +183,7 @@ void check_packets(u_char *pingdata, const struct pcap_pkthdr *header, const uns
       }
     } else
       if (type == NXT_UDP && ptr[nxt] == NXT_UDP)
-        printf("%04u.%04ld \tudp", (int) (ts2.tv_sec - ts.tv_sec - min), usec);
+        printf("%04u.%03ld \tudp", (int) (ts2.tv_sec - ts.tv_sec - min), usec);
   }
   if (resp_type >= 0)
     printf(" packet received from %s%s\n", thc_ipv62notation(ptr + 8), frag);
@@ -563,7 +564,7 @@ int main(int argc, char *argv[]) {
           usleep(1);
     clock_gettime(CLOCK_REALTIME, &ts);
     if (flood < 2) {
-      printf("0000.0000 \t%s packet sent to %s\n", port == 0 ? "ping" : type == NXT_TCP ? "tcp-syn" : "udp", thc_ipv62notation(dst6));
+      printf("0000.000 \t%s packet sent to %s\n", port == 0 ? "ping" : type == NXT_TCP ? "tcp-syn" : "udp", thc_ipv62notation(dst6));
       if (flood == 1 && type != NXT_TCP && type != NXT_UDP && frag == 0) {
         if (xl)
           for (i = 0; i < count; i++)
